package top.xinsin.service;

import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.User;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import top.xinsin.mapper.AccountMapper;
import top.xinsin.pojo.Account;
import top.xinsin.util.HttpCodes;
import top.xinsin.util.R;

import java.util.Date;
import java.util.List;

/**
 * @author xinsin
 * Created On 2023/4/24 21:11
 * @version 1.0
 */
@Service
@Slf4j
public class AccountService implements UserDetailsService {
    @Autowired
    private AccountMapper accountMapper;
    BCryptPasswordEncoder encoder = new BCryptPasswordEncoder();

    public AccountService() {
    }

    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        System.out.println(username);
        if (username == null) {
            throw new UsernameNotFoundException("用户名不能为空");
        } else {
            Account account = this.accountMapper.selectOne(new LambdaQueryWrapper<Account>().eq(Account::getUsername, username).or().eq(Account::getEmail, username).eq(Account::getStatus, true));
            if (account == null) {
                throw new UsernameNotFoundException("用户名或密码错误");
            } else {
                return User.withUsername(account.getUsername()).password(account.getPassword()).roles(new String[]{"role=" + account.getRoleId(), "id=" + account.getId()}).build();
            }
        }
    }

    public R<String> register(Account account) {
        account.setCreateTime(new Date());
        account.setStatus(true);
        account.setPassword(this.encoder.encode(account.getPassword()));
        int insert = this.accountMapper.insert(account);
        return insert == 1 ? R.success("注册成功") : R.failed(HttpCodes.HTTP_CODES555, "注册失败,您的用户名或者邮箱已经被注册过了");
    }

    public R<String> changeAccount(Account account) {
        if (account.getPassword() != null) {
            account.setPassword(this.encoder.encode(account.getPassword()));
        }

        int update = this.accountMapper.updateById(account);
        return update == 1 ? R.success("修改用户信息成功") : R.failed(HttpCodes.HTTP_CODES555, "修改用户信息失败,请联系服务器管理员");
    }

    public R<String> changeStatus(Integer userId, Boolean status) {
        Account account = new Account();
        account.setId(userId);
        account.setStatus(status);
        int i = this.accountMapper.updateById(account);
        if (i == 1) {
            return status ? R.success("启用该用户成功") : R.success("禁用该用户成功");
        } else {
            return R.failed(HttpCodes.HTTP_CODES555, "修改用户状态失败");
        }
    }

    public R<String> changePassword(Integer userId, String oldPassword, String newPassword) {
        Account account = new Account();
        account.setPassword(this.encoder.encode(newPassword));
        LambdaQueryWrapper<Account> accountLambdaQueryWrapper = new LambdaQueryWrapper<>();
        accountLambdaQueryWrapper.eq(Account::getId, userId).eq(Account::getPassword, this.encoder.encode(oldPassword));
        int update = this.accountMapper.update(account, accountLambdaQueryWrapper);
        return update == 1 ? R.success("修改密码成功") : R.failed(HttpCodes.HTTP_CODES555, "修改密码失败,请联系服务器管理员");
    }

    public R<JSONObject> getUser(Integer page, Integer num) {
        Page<Account> accountPage = new Page<>((long)page, (long)num);
        Page<Account> accountPage1 = this.accountMapper.selectPage(accountPage, null);
        JSONObject jsonObject = new JSONObject();
        List<Account> collect = accountPage1.getRecords().stream().peek((e) -> {
            e.setPassword((String)null);
        }).toList();
        jsonObject.fluentPut("info", collect).fluentPut("total", accountPage1.getTotal());
        return R.success(jsonObject);
    }

    public R<String> delUser(Integer id) {
        int delete = this.accountMapper.deleteById(id);
        return delete == 1 ? R.success("删除用户成功") : R.failed(HttpCodes.HTTP_CODES555, "删除用户失败,请联系服务器管理员");
    }
}
